/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "gctl/core.h"
#include "gctl/io.h"
#include "../lib/potential.h"
// 多线程异步头文件
#include <thread>
#include <future>

using namespace gctl;

shc_data sd;
array<point3ds> obsp;

// Read SHC coefficients
int read_shc_file()
{
    dsv_io shc_in;
    shc_in.delimeter(',');
    shc_in.load_text("data/gggrx/gggrx_0660pm_sha", ".tab");
    shc_in.info(ColInfo);

    _1d_array gnm, hnm;
    shc_in.get_column(gnm, 3);
    shc_in.get_column(hnm, 4);

    sd.init(gnm, hnm, 1, 0, 660, 660, 1738.0);
    sd.set_GM(4902.799806931690);

    std::cout << "Done reading SHC coefficients\n";
    return 0;
}

// Prepare observation sites
int init_observations()
{
    grid_points_2d(obsp, -179.5, 179.5, -89.5, 89.5, 1.0, 1.0, 1738.0 + 100.0, TopLeft);

    std::cout << "Done initializing obervations points\n";
    return 0;
}

// Main process
int main(int argc, char *argv[]) try
{
    std::future<int> future1 = std::async(std::launch::async, read_shc_file);
    std::future<int> future2 = std::async(std::launch::async, init_observations);

    int result1 = future1.get();
    int result2 = future2.get();
    if (result1 != 0 || result2 != 0)
        throw std::runtime_error("Terminated with unexpected errors.");

    // Calculate over all observation sites
    array<point3dc> V;
    gobser(V, sd, obsp, FullMsg);

    array<double> Vr = V.extract<double>([](const point3dc &b) -> double {return b.z;});
    array<double> Vt = V.extract<double>([](const point3dc &b) -> double {return b.x;});
    array<double> Vp = V.extract<double>([](const point3dc &b) -> double {return b.y;});
    Vr.scale(1e+8);
    Vt.scale(1e+5);
    Vp.scale(1e+5);

    // Save resutls
    geodsv_io magout;
    magout.init_table(obsp.size(), 6);
    magout.column_names({"lon", "lat", "rad", "Vr", "Vt", "Vp"});
    magout.fill_column_point3ds(obsp, "rad", "lon", "lat");
    magout.fill_column(Vr, "Vr");
    magout.fill_column(Vt, "Vt");
    magout.fill_column(Vp, "Vp");
    magout.save_csv("data/gggrx/gggrx_0660pm_sha_100km_grid");

    // Create netcdf files
    save_netcdf_grid("data/gggrx/gggrx_0660pm_sha_100km_grid", Vr, 360, 180, -179.5, 1.0, -89.5, 1.0, TopLeft, "lon", "lat", "Vr");
    append_netcdf_grid("data/gggrx/gggrx_0660pm_sha_100km_grid", Vt, "lon", "lat", "Vt", TopLeft);
    append_netcdf_grid("data/gggrx/gggrx_0660pm_sha_100km_grid", Vp, "lon", "lat", "Vp", TopLeft);
    return 0;
}
catch (std::exception &e)
{
    GCTL_ShowWhatError(e.what(), GCTL_ERROR_ERROR, 0, 0, 0);
}